import type { HChild } from 'hastscript/lib/core.js';
import { HighlighterOptions } from 'shiki';
import type { HPropertyValue } from 'hastscript/lib/core.js';
import type { HResult } from 'hastscript/lib/core.js';
import type { HStyle } from 'hastscript/lib/core.js';
import type { IThemedToken } from 'shiki';
import type { Root } from 'hast';

/**
 * Create MDsveX code highlighting function.
 *
 * @param options - Shiki highlighter options object.
 * @returns A Promise of a MDsvex highlighter function.
 * @public
 */
export declare function createHighlighter(options: HighlighterOptions): Promise<MdvexHighlighter>;

/**
 * `code` tag renderer props.
 * @public
 */
declare type ICodeRenderProps = IRenderProps;

/**å
 * @public
 */
export declare interface IHastRendererOptions {
    /**
     * Directive to include a language identifier in the rendered output.
     */
    langId?: string;
    /**
     * Optional foreground color for `code` elements that do not have a grammar.
     */
    fg?: string;
    /**
     * Optional background color of `code` elements.
     */
    bg?: string;
    /**
     * Optional classes for specific line numbers.
     */
    lineOptions?: ILineOption[];
    /**
     * Optional elements override functions.
     *
     * Allows for fine grained control overrides for the rendering of `pre` and `code`
     * tags, lines, or individual tokens.
     */
    elements?: Partial<IRenderOptions>;
    /**
     * Shiki theme name.
     */
    themeName?: string;
}

/**
 * Optional {@link IHastRendererOptions} property, allows inclusion of classes for
 * specific line numbers.
 *
 * @public
 */
export declare interface ILineOption {
    /**
     * 1-based line number.
     */
    line: number;
    /**
     * Class name/names to include.
     */
    className?: HPropertyValue;
}

/**
 * Line renderer props.
 *
 * @public
 */
declare interface ILineRenderProps extends IRenderProps {
    /**
     * Class name/names of the line to be rendered, as generated from a combination of
     * the {@link LineOptions} and the defaults.
     */
    className?: HPropertyValue;
    /**
     * The entire Shiki code block of tokens.
     */
    lines: readonly IThemedToken[][];
    /**
     * The Shiki tokens of the line.
     */
    line: readonly IThemedToken[];
    /**
     * Zero based index of the line number.
     */
    index: number;
}

/**
 * `pre` tag renderer props.
 *
 * @public
 */
declare interface IPreRenderProps extends IRenderProps {
    /**
     * Class name/names of the line to be rendered, as generated from a combination of
     * the {@link LineOptions} and the defaults.
     */
    className?: HPropertyValue;
    /**
     * Styles applied to the token by Shiki.
     *
     * Should be included in the {@link hastscript#h  | h} {@link hastscript#Properties | props} argument.
     */
    style?: HStyle;
}

/**
 * Options to modify the syntax tree of the hast code block.
 *
 * @public
 */
export declare interface IRenderOptions {
    /**
     * Override function for `pre` tags.
     */
    readonly pre: (props: IPreRenderProps) => HResult;
    /**
     * Override function for `code` tags.
     */
    readonly code: (props: ICodeRenderProps) => HResult;
    /**
     * Override function for code block lines.
     */
    readonly line: (props: ILineRenderProps) => HResult;
    /**
     * Override function for code block tokens.
     */
    readonly token: (props: ITokenRenderProps) => HResult;
}

/**
 * Common renderer props.
 *
 * @public
 */
declare interface IRenderProps {
    /**
     * Children of the renderer.
     *
     * IMPORTANT: It is necessary pass children down through the renderer, otherwise
     * the code block will not be properly rendered.
     */
    children: HChild;
    /**
     * Additional information about the code block.
     */
    meta: Metadata;
    [key: string]: unknown;
}

/**
 * @public
 */
export declare interface IRenderToHastParams {
    /**
     * Optional code block metadata.
     */
    metadata?: Metadata;
    /**
     *  Options to modify the render function.
     */
    options?: IHastRendererOptions;
    /**
     * Shiki highlighted tokens to render.
     */
    tokens: IThemedToken[][];
}

/**
 * Token renderer props.
 *
 * @public
 */
declare interface ITokenRenderProps extends IRenderProps {
    /**
     * Styles applied to the token by Shiki.
     *
     * Should be included in the {@link hastscript#h | h} {@link hastscript#Properties | props} argument.
     */
    style: string;
    /**
     * An array of the entire line of tokens.
     */
    tokens: readonly IThemedToken[];
    /**
     * The Shiki {@link shiki#IThemedToken| token} to render.
     */
    token: IThemedToken;
    /**
     * Zero-based index of the token position in the line.
     */
    index: number;
}

/**
 * MDsveX highlighter type.
 *
 * NOTE: not exported from `mdsvex`, so its copied here.
 * @public
 */
export declare type MdvexHighlighter = (code: string, lang: string | undefined, metastring?: string | undefined) => string | Promise<string>;

/**
 * Markdown Code Block Metadata
 *
 * @public
 */
export declare type Metadata = {
    /**
     * The lines to highlight.
     */
    lineNumbers: number[];
    /**
     * The initial line number to start from.
     */
    lineNumbersStart: number;
    /**
     * Boolean indicating where or not to show line numbers.
     */
    showLineNumbers: boolean;
    /**
     * The title of the code block.
     */
    title?: string | undefined;
    /**
     * The language of the code block.
     */
    lang?: string | undefined;
};

/**
 * Parse Markdown code fence metadata.
 *
 * @param metastring - Code block metadata string.
 * @returns Parsed metadata values object.
 * @public
 */
export declare function parseMetadata(metastring: string | undefined): Metadata;

/**
 * Render Shiki tokens to hast, Hypertext Abstract Syntax Tree format.
 *
 * @param renderParams - The render {@link IRenderToHastParams | parameters}
 * @returns A hast syntax tree for a highlighted code block.
 * @public
 */
export declare function renderToHast({ metadata, options, tokens, }: IRenderToHastParams): Root;

export { }
